package com.ibm.iru.db2express9_1lnx;

/**
 * @(#)DB2Express9_1LnxPDC.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the IBM Express Runtime.
 */

import java.io.File;

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * This class checks that DB2 Express 9.1 can be installed
 * under the specified installation directory, with the specified options. 
 *
 */
public class DB2Express9_1LnxPDC extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  
  private SupportLinuxHelper ivHelper;
  private String ivVersion = null;
  private DB2InstalledProduct db2Product;

  /**
   * Process the arguments args[0] = response file name
   * @return SUCCESS if successful otherwise return FAILURE
   */
  private int init(String args[])
  {
    int rc = SUCCESS;

    setMainResources("com.ibm.iru.message.MessagesNLS");
    int no_Of_Expected_args = 1;
    ivHelper = getLinuxHelper();

    //if an incorrect number of arguments were passed log the error.
    if (no_Of_Expected_args != args.length)
    {
      String[] replacementText = { "DB2Express9_1LnxPDC", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.log(this);
      rc = FAILURE;
    }
    else
    {
      setResponseFileName(args[0].trim());
      ivVersion = ivHelper.getProductInstallingVersion(this);
    }

    return rc;
  }

  /**
   * Determine install action:
   * 
   * @return PDC_DOES_NOT_EXIST - if specified install directory does not exist, or is empty
   *         PDC_EXIST -          if same version already installed
   *         FAILURE -            Another version of DB2 is already installed in the destinarion directory, 
   *                              specified directory is not empty, or install location is not specified  
   */
  private int determineInstallAction()
  {
    int returnCode = PDC_DOES_NOT_EXIST;
    String versionInstalled = null;
    setKey("FILE");
    String installDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    //if no value then return FAILURE as installation location is a required parameter
    if ((installDir == null) || installDir.equals(""))
    {
      setMessage(getResourceString(NLSKeys.INSTALLATION_LOCATION_NOT_FOUND));
      ivHelper.postErrorMessageToDeployerUI(this);	
      return FAILURE;
    }

    //check if specified installation directory exists
    setFileName(installDir);
    if (ivHelper.fileExists(this))
    {
      //specified installation directory exists, check if it is empty
      File file = new File(installDir);
      if ((file.list()).length == 0)
      {
        //location is emtpy
        returnCode = PDC_DOES_NOT_EXIST;
      }
      else
      {        
        //check if DB2 is installed 
        db2Product = DB2InstalledProduct.getInstalledProduct(this, installDir);
        if (db2Product == null)
        {
          setMessage(getResourceString(NLSKeys.NONEMPTY_DIR));
          ivHelper.postErrorMessageToDeployerUI(this);
          returnCode = FAILURE;
        }
        else
        {
          //get version installed 
          versionInstalled = db2Product.getInstalledVersion().trim();
          //check same version of DB2 is already installed
          if (versionInstalled.equals(ivVersion))
          {
            //Current version of DB2 is already installed
            returnCode = PDC_EXISTS;
          }
          else
          {
            setMessage(getResourceString(NLSKeys.OTHER_DB2_COPY_EXIST, versionInstalled));
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
          }
        }
      }
    }
    return returnCode;
  }

  /**
   * Conditional and environmental variables are verified.
   * <li> Make sure all the required variables contain a value.
   * <li> Check that if any of the four fenced variables are specified, all others 
   *   must be specified too.
   * <li> Check that if any of the four DAS variables are specified, all others
   *   must be specified too.
   * <li> Check that if provided DAS user name and password, they match if they exist.
   * <li> Check that if DAS user name is specified, it is different from the instance
   *   user name. If equal, a FAILURE will be returned.
   * <li> Check that DB2 instance user name and password match (in case they exist).
   * <li> Check that instance user specified does not exist as an instance user for another
   * DB2 copy.
   * 
   * Any problems found will be logged.
   * 
   * @return true = no problems found. false = problem found.
   */
  private boolean validDeploymentParameters()
  {
    boolean validity = true;

    //DB2 instance user name, password, group name, and home dir are required
    setKey("db2inst1.NAME");
    String instanceUserName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("db2inst1.PASSWORD");
    String instanceUserPassword = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("db2inst1.GROUP_NAME");
    String instanceGroupName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("db2inst1.HOME_DIRECTORY");
    String instanceHomeDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    if (instanceUserName == null || instanceUserName.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_USER));
      validity = false;
      ivHelper.postErrorMessageToDeployerUI(this);
    }
    
    if (instanceUserPassword == null || instanceUserPassword.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_PASSWORD));
      validity = false;
        ivHelper.postErrorMessageToDeployerUI(this);
    }

    if (instanceGroupName == null || instanceGroupName.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_GROUP));
      validity = false;
        ivHelper.postErrorMessageToDeployerUI(this);
    }

    if (instanceHomeDir == null || instanceHomeDir.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_HOMEDIR));
      validity = false;
      ivHelper.postErrorMessageToDeployerUI(this);
    }

    if (validity == true)
      validity = checkUserNamePassword(instanceUserName, instanceUserPassword);
    // Although DAS user configuration properties are required and this code is not needed, 
    // in the future these properties may not be compulsory and this section of code could
    // utilized again.
    
    //check that if provided DAS user name and password, they match if they exist
    String dasUserName = ivHelper.getIbmNsiPropValue("dasUserName");
    String dasUserPass = ivHelper.getPasswordValue("dasUserPassword");
    String dasGroup = ivHelper.getIbmNsiPropValue("dasGroup");
    String dasHomeDir = ivHelper.getIbmNsiPropValue("dasHomeDir");
    
    boolean isDasUserNameEmpty = (dasUserName == null || dasUserName.equals(""));
    boolean isDasPasswordEmpty = (dasUserPass == null || dasUserPass.equals(""));
    boolean isDasGroupNameEmpty = (dasGroup == null || dasGroup.equals(""));
    boolean isDasHomeDirEmpty = (dasHomeDir == null || dasHomeDir.equals(""));
    
    //make sure that DAS variables are all empty or all have data.
    if (!((isDasUserNameEmpty && isDasPasswordEmpty && isDasGroupNameEmpty && isDasHomeDirEmpty) 
        || (!isDasUserNameEmpty && !isDasPasswordEmpty && !isDasGroupNameEmpty && !isDasHomeDirEmpty)))
    {
      validity = false;
      setMessage(getResourceString(NLSKeys.DASUSER_MISSING_VALUES));
      ivHelper.postErrorMessageToDeployerUI(this);
    }
    
    if (!isDasUserNameEmpty && !isDasPasswordEmpty)
    {
      if (!checkUserNamePassword(dasUserName, dasUserPass))
        validity = false;
      else
      {
        //make sure it is different from the instance user name
        if (dasUserName.equalsIgnoreCase(instanceUserName))
        {
          validity = false;
          setMessage(getResourceString(NLSKeys.DB2_DAS_USER_EQUAL_INSTANCE_USER));
          ivHelper.postErrorMessageToDeployerUI(this);
        }
      }
    }

    setKey("db2inst1.FENCED_USERNAME");
    String fencedUserName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("db2inst1.FENCED_PASSWORD");
    String fencedUserPass = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("db2inst1.FENCED_GROUP_NAME");
    String fencedGroupName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("db2inst1.FENCED_HOME_DIRECTORY");
    String fencedHomeDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    boolean isFencedUserNameEmpty = (fencedUserName == null || fencedUserName.equals(""));
    boolean isFencedPasswordEmpty = (fencedUserPass == null || fencedUserPass.equals(""));
    boolean isFencedGroupNameEmpty = (fencedGroupName == null || fencedGroupName.equals(""));
    boolean isFencedHomeDirEmpty = (fencedHomeDir == null || fencedHomeDir.equals(""));

    //make sure that fenced variables are all empty or all have data.
    if (!((isFencedUserNameEmpty && isFencedPasswordEmpty && isFencedGroupNameEmpty && isFencedHomeDirEmpty) 
        || (!isFencedUserNameEmpty && !isFencedPasswordEmpty && !isFencedGroupNameEmpty && !isFencedHomeDirEmpty)))
    {
      validity = false;
      setMessage(getResourceString(NLSKeys.FENCEDUSER_MISSING_VALUES));
      ivHelper.postErrorMessageToDeployerUI(this);

    }
    //check that if provided fenced user name and password, they match if they exist
    if (!(isFencedUserNameEmpty || isFencedPasswordEmpty))
    {
      if (!checkUserNamePassword(fencedUserName, fencedUserPass))
        validity = false;
    }

    if (validity == true)
    {
      //check that instance user is not an instance user for another DB2 copy
      DB2InstalledProduct[] db2Products = DB2InstalledProduct.getInstalledProducts(this);
      //for each DB2 product get the instance user name and compare with the one given by the user      
      if (db2Products != null)
      {
        for (int i=0; i<db2Products.length; i++)
        {
          if(instanceUserName.equals(db2Products[i].getLinuxInstanceUser()))
          {
            validity = false;
            setMessage(getResourceString(NLSKeys.DB2_INSTANCE_USER_ALREADY_EXISTS, db2Products[i].getLinuxInstanceUser()));
            ivHelper.postErrorMessageToDeployerUI(this);
            break;
          }
        }
      }
    }
    return (validity);
  }

  /**
   * Ensures the user ID and password match if they exist.
   *
   * @param String username, String password to check
   * @return true  - no user is found or the password for the user is correct
   *         false - the password is invalid for the given user.
   */
  private boolean checkUserNamePassword(String userName, String password)
  {
    Boolean validUserPassword = true;
    //Check the user
    setUserName(userName);
    if (getLinuxHelper().doesUserExist(this))
    {
      setPassword(password);
      if (!getLinuxHelper().isUserAndPasswordValid(this))
      {
        validUserPassword = false;
        setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, userName));
        getHelper().postErrorMessageToDeployerUI(this);         
      }
    }

    return validUserPassword;
  }

  /**
   * Perform required checks before installing DB2.
   * Need to check if installation directory specified exists and validate variables.
   * @return PDC_EXISTS, PDC_DOES_NOT_EXIST or FAILURE
   */
  private int performPredeploymentChecks()
  {
    int rc = determineInstallAction();
    
    // Check to see that the install will take place before validating deployment parameters. 
    if (rc != PDC_EXISTS && rc != FAILURE)
    {
      if (!validDeploymentParameters())
      {
        if( rc == PDC_DOES_NOT_EXIST)
        {
          //override installAction
          rc = FAILURE;
        }
      }
    }
    return rc;
  }

  /**
   * Entry point
   */
  public static void main(String args[])
  {
    // Create and initialize the object
    DB2Express9_1LnxPDC pdc = new DB2Express9_1LnxPDC();
    int rc = pdc.init(args);

    if (rc == SUCCESS)
    {
      // Check if DB2 is already installed and validate variables
      rc = pdc.performPredeploymentChecks();      
    }

    System.exit(rc);
  }
}